//---------------------------------------------------------------------------
#include <vcl\vcl.h>
#include <GL\GL.h>
#pragma hdrstop

#include "BCGLVP.h"
//---------------------------------------------------------------------------
static inline TOpenGLViewport* ValidCtrCheck()
{
        return new TOpenGLViewport(NULL);
}
//---------------------------------------------------------------------------
__fastcall TOpenGLViewport::TOpenGLViewport(TComponent* Owner)
        : TWinControl(Owner)
        , rc(0)
        , dc(0)
        , shareVP(0)
{
        Width = 192;
        Height = 128;
}
//---------------------------------------------------------------------------
void __fastcall TOpenGLViewport::CreateContext()
{
        PIXELFORMATDESCRIPTOR pfd = {0};
        int pf;

        // Try to acquire a device context from the underlying window
        dc = GetDC(Handle);
        if (!dc) goto fail;

        // Find a suitable PFD
        pfd.nSize = sizeof(pfd);
        pfd.nVersion = 1;
        pfd.dwFlags = PFD_DRAW_TO_WINDOW|PFD_SUPPORT_OPENGL|PFD_DOUBLEBUFFER;
        pfd.iPixelType = PFD_TYPE_RGBA;
        pfd.cColorBits = 24;
        pfd.cAlphaBits = 8;
        pfd.cDepthBits = 24;
        pfd.cStencilBits = 8;
        pf = ChoosePixelFormat(dc, &pfd);
        if (!pf) goto fail;

        // Set pixel format
        if (!SetPixelFormat(dc, pf, &pfd)) goto fail;

        // Create GL rendering context
        rc = wglCreateContext(dc);
        if (!rc) goto fail;

        // Share resources, if set
        if (shareVP)
        {
                wglShareLists(rc, shareVP->rc);
                shareVP = 0;
        }

        // Success!
        return;
        
    fail:
        DestroyContext();
}
//---------------------------------------------------------------------------
void __fastcall TOpenGLViewport::DestroyContext()
{
        if (rc) wglDeleteContext(rc);
        if (dc) ReleaseDC(Handle, dc);
        rc = 0;
        dc = 0;
}
//---------------------------------------------------------------------------
void __fastcall TOpenGLViewport::WMDestroy(TMessage& Message)
{
        DestroyContext();
}
//---------------------------------------------------------------------------
void __fastcall TOpenGLViewport::WMEraseBkgnd(TMessage& Message)
{
        // Do not erase the background to avoid flickering
        Message.Result = 0;
}
//---------------------------------------------------------------------------
void __fastcall TOpenGLViewport::WMPaint(Messages::TWMPaint& Message)
{
        // For design mode, just draw some generic background
        if (ComponentState.Contains(csDesigning))
        {
                // Draw rectangle
                HDC dc = GetDC(Handle);
                HGDIOBJ previous = ::SelectObject(dc, GetStockObject(GRAY_BRUSH));
                Rectangle(dc, 0, 0, Width, Height);
                ::SelectObject(dc, previous);
                ReleaseDC(Handle, dc);

                // Validate window rect
                ValidateRect(Handle, 0);

                // Message handled
                Message.Result = 0;
                return;
        }

        // Create context if necessary
        if (!rc) CreateContext();

        // Activate context
        if (MakeCurrent())
        {
                // Update viewport
                glViewport(0, 0, Width, Height);

                // Call render code
                if (FOnRender)
                {
                    FOnRender(this);
                }
                else // no render code, just clear the background
                {
                    glClearColor(0, 0, 0, 0);
                    glClear(GL_COLOR_BUFFER_BIT);
                }
                
                // Swap buffers
                SwapBuffers();
        }

        // Validate window rect
        ValidateRect(Handle, 0);

        // Message handled
        Message.Result = 0;
}
//---------------------------------------------------------------------------
void __fastcall TOpenGLViewport::ShareResourcesWith(TOpenGLViewport* vp)
{
        if (rc) throw new Exception("Cannot share resources after the context has been created");
        shareVP = vp;
}
//---------------------------------------------------------------------------
bool __fastcall TOpenGLViewport::MakeCurrent() const
{
        return wglMakeCurrent(dc, rc) == TRUE;
}
//---------------------------------------------------------------------------
void __fastcall TOpenGLViewport::SwapBuffers() const
{
        ::SwapBuffers(dc);
}
//---------------------------------------------------------------------------
namespace Bcglvp
{
        void __fastcall Register()
        {
                TComponentClass classes[1] = {__classid(TOpenGLViewport)};
                RegisterComponents("Additional", classes, 0);
        }
}
//---------------------------------------------------------------------------
