//---------------------------------------------------------------------------
#include <vcl\vcl.h>
#include <GL/GL.h>
#include <GL/GLu.h>
#pragma hdrstop

#include "Unit1.h"
//---------------------------------------------------------------------------
#pragma link "BCGLVP"
#pragma resource "*.dfm"
TForm1 *Form1;
//---------------------------------------------------------------------------
__fastcall TForm1::TForm1(TComponent* Owner)
        : TForm(Owner)
{
        // Hook idle handler
        Application->OnIdle = AppIdle;
}
//---------------------------------------------------------------------------
void __fastcall TForm1::AppIdle(TObject* Sender, bool& Done)
{
        // This causes the viewport to be drawn repeatedly
        OpenGLViewport1->Invalidate();
        Done = false;
}
//---------------------------------------------------------------------------
static void DrawPlanet(float rotation, float radius, int limit)
{
        // This planet
        GLUquadricObj* sphere = gluNewQuadric();
        gluQuadricNormals(sphere, GLU_SMOOTH);
        gluQuadricTexture(sphere, GL_FALSE);
        if (Form1->cbWireframe->Checked)
                gluQuadricDrawStyle(sphere, GLU_LINE);
        else
                gluQuadricDrawStyle(sphere, GLU_FILL);
        gluSphere(sphere,
                radius,
                atof(Form1->edSlices->Text.c_str()),
                atof(Form1->edStacks->Text.c_str()));
        gluDeleteQuadric(sphere);

        // More planets
        if (limit > 0)
        {
                for (int i=0; i < limit; ++i)
                {
                        glPushMatrix();
                        glRotatef(360/limit*i, 0, 1, 0);
                        glTranslatef(radius*2.5, 0, 0);
                        glRotatef(rotation*2, 0, 1, 0);
                        DrawPlanet(rotation*1.2, radius*0.4, limit - 1);
                        glPopMatrix();
                }
        }
}
//---------------------------------------------------------------------------
void __fastcall TForm1::OpenGLViewport1Render(TObject *Sender)
{
        unsigned __int64 start, end, freq;

        // Capture before frame time
        QueryPerformanceFrequency((_LARGE_INTEGER*)&freq);
        QueryPerformanceCounter((_LARGE_INTEGER*)&start);

        // Use time as rotation
        static float rotation = 0;
        if (cbRotate->Checked) rotation = (float)start/(float)freq*50.0f;

        // Render scene
        glClearColor(0.1, 0.11, 0.12, 1);
        glClear(GL_COLOR_BUFFER_BIT|GL_DEPTH_BUFFER_BIT);
        glEnable(GL_DEPTH_TEST);
        glMatrixMode(GL_PROJECTION);
        glLoadIdentity();
        gluPerspective(30, (float)OpenGLViewport1->Width/(float)OpenGLViewport1->Height, 0.1, 100);
        glMatrixMode(GL_MODELVIEW);
        glLoadIdentity();
        if (Form1->cbLighting->Checked)
        {
                const GLfloat backlightPos[4][4] = {{0, 0.8, -1, 0},
                                                    {0, -0.8, -1, 0},
                                                    {0.8, 0, -1, 0},
                                                    {-0.8, 0, -1, 0}};
                const GLfloat backlightDiffuse[4] = {0.4, 0.8, 1, 1};
                const GLfloat worldlightDiffuse[4] = {0.4, 0.5, 0.6, 1};
                const GLfloat worldlightPosition[4] = {1, 0, 1, 0};
                const GLfloat sunlightDiffuse[4] = {0.7, 0.55, 0.35, 1};
                const GLfloat sunlightPosition[4] = {-1, 0.5, 1, 0};
                glEnable(GL_LIGHTING);
                for (int i=0; i < 4; ++i)
                {
                        glEnable(GL_LIGHT0 + i);
                        glLightfv(GL_LIGHT0 + i, GL_POSITION, backlightPos[i]);
                        glLightfv(GL_LIGHT0 + i, GL_DIFFUSE, backlightDiffuse);
                        glLightfv(GL_LIGHT0 + i, GL_SPECULAR, backlightDiffuse);
                }
                glEnable(GL_LIGHT4);
                glLightfv(GL_LIGHT4, GL_POSITION, worldlightPosition);
                glLightfv(GL_LIGHT4, GL_DIFFUSE, worldlightDiffuse);
                glEnable(GL_LIGHT5);
                glLightfv(GL_LIGHT5, GL_POSITION, sunlightPosition);
                glLightfv(GL_LIGHT5, GL_DIFFUSE, sunlightDiffuse);
                glEnable(GL_LIGHT6);
                glLightfv(GL_LIGHT6, GL_POSITION, sunlightPosition);
                glLightfv(GL_LIGHT6, GL_DIFFUSE, sunlightDiffuse);
        }
        else
        {
                glDisable(GL_LIGHTING);
        }
        glTranslatef(0, 0, -22);
        glRotatef(25, 1, 0, 0);
        glRotatef(25, 0, 0, 1);
        glRotatef(rotation, 0, 1, 0);
        DrawPlanet(rotation, atof(edRadius->Text.c_str()), 4);

        // Capture after frame time
        QueryPerformanceCounter((_LARGE_INTEGER*)&end);

        // Report speed
        int fps = end > start ? (int)((float)freq/(float)(end - start)) : 999999;
        StatusBar1->SimpleText = IntToStr(fps) + " fps";
}
//---------------------------------------------------------------------------
